PUZZLE_TEMPLATE_ID = "1drSoyrE4gM3JaGweDkOybwXWdKPIDTfUmB1gQCYS3Uw"
PUZZLE_TEMPLATE_SHEETS = ["Text", "Grid"]

def sheets_create(turb, name):
    """Create a new sheet with the given name.

    Returns the dict with 'id' and 'url'of the spreadsheet.
    """

    # Create a new spreadsheet
    spreadsheet_body = {
        'properties': {
            'title': name
        }
    }

    new_sheet = turb.sheets.create(body=spreadsheet_body).execute()

    # Now that we've created a new sheet, we need to also allow anyone
    # with the link to the sheet to be able to edit it.
    turb.permissions.create(fileId=new_sheet["spreadsheetId"],
                            body={'type': 'anyone', 'role': 'writer'},
                            fields='id').execute()

    return {
        'id': new_sheet["spreadsheetId"],
        'url': new_sheet["spreadsheetUrl"]
    }

def sheets_create_for_puzzle(turb, puzzle):
    """Creates a new sheet for a puzzle of the given name

    Like sheets_create(), but also copies the puzzle template sheet.

    Here, 'puzzle' is a dict that must have a 'name' key and may optionally
    have a 'channel_url' or 'url' key."""

    # First create the new sheet
    new_sheet = sheets_create(turb, puzzle['name'])

    # Insert some useful links into the sheet
    url_link=''
    if 'url' in puzzle:
        url_text = "Original puzzle is at: {}".format(puzzle['url'])
        url_link = puzzle['url']
    else:
        url_text = ''

    channel_url_link = ''
    if 'channel_url' in puzzle:
        channel_url_text = "Discussion for this puzzle is at: {}".format(
            puzzle['channel_url'])
        channel_url_link = puzzle['channel_url']
    else:
        channel_url_text = ''

    turb.sheets.values().append(
        spreadsheetId=new_sheet['id'],
        range='A1:A2',
        valueInputOption='USER_ENTERED',
        insertDataOption='INSERT_ROWS',
        body={
            'range': 'A1:A2',
            'values': [
                ['=HYPERLINK("'+url_link+'","'+url_text+'")'],
                ['=HYPERLINK("'+channel_url_link+'","'+channel_url_text+'")']
            ]
        }).execute()

    # Copy some sheets from the Template spreadsheet

    response = turb.sheets.get(spreadsheetId=PUZZLE_TEMPLATE_ID).execute()

    for sheet in response["sheets"]:
        if sheet["properties"]["title"] in PUZZLE_TEMPLATE_SHEETS:
            turb.sheets.sheets().copyTo(spreadsheetId=PUZZLE_TEMPLATE_ID,
                                        sheetId=sheet["properties"]["sheetId"],
                                        body={
                                            "destinationSpreadsheetId":
                                            new_sheet['id']
                                        }).execute()

    return new_sheet

def renameSheet(turb, url, newName):
    id = extractIdFromSheetUrl(url)
    requests = []
    requests.append({
        'updateSpreadsheetProperties': {
            'properties': {
                'title': newName
            },
            'fields': 'title'
        }
    })

    body = {
        'requests': requests
    }

    turb.sheets.batchUpdate(spreadsheetId = id,
                            body=body
                            ).execute()

def extractIdFromSheetUrl(url):
    # Google sheet ids are between the /d/ and /edit in the url, like
    # https://docs.google.com/spreadsheets/d/1dxHBzjen...-LaXeVPrg/edit#gid=0
    startIndex = url.find('/d/') + 3
    endIndex = url.find('/edit')
    return url[startIndex : endIndex]
